﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using IdeaBlade.Core;
using IdeaBlade.EntityModel;
using System.Collections.Specialized;
using System.Linq;
using System.Collections.Generic;

namespace SimpleSteps {
  public class MainPageViewModel {
    public ObservableCollection<Employee> Employees { get; private set; }
    public ObservableCollection<string> Log { get; private set; }
    private NorthwindIBEntities _mgr;

    public MainPageViewModel() {
      AsyncMainPageViewModel();
    }

    async void AsyncMainPageViewModel() {
      Log = new ObservableCollection<string>();
      Employees = new ObservableCollection<Employee>();
      Employees.CollectionChanged += Employees_CollectionChanged;
      _mgr = new NorthwindIBEntities();

      _mgr.Fetching += (s, e) => WriteToLog("Fetching " + e.Query.ElementType);  //lambda syntax
      _mgr.Queried += (s, e) => WriteToLog("Query returned " + e.Results.Cast<object>().Count());

      var query = _mgr.Employees.Include("Orders");
      var results = await query.ExecuteAsync();
      results.ForEach(Employees.Add);
    }

    public void Save() {
      _mgr.SaveChangesAsync();
    }

    private void Employees_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e) {
      switch (e.Action) {
        case NotifyCollectionChangedAction.Add:
          Add(e.NewItems.Cast<Employee>());
          break;
        case NotifyCollectionChangedAction.Remove:
          Delete(e.OldItems.Cast<Employee>());
          break;
      }
    }

    private void Add(IEnumerable<Employee> employees) {
      employees.ForEach(_mgr.AddEntity);
    }

    private void Delete(IEnumerable<Employee> employees) {
      employees.ForEach(employee => employee.EntityAspect.Delete());
      Save();
    }

    private void WriteToLog(string message) {
      Log.Insert(0, message);
    }


  }

}
