﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using IdeaBlade.Core;
using IdeaBlade.EntityModel;

namespace SimpleSteps {
  public class MainPageViewModel : INotifyPropertyChanged {


    private NorthwindIBEntities _mgr = new NorthwindIBEntities();
    public ObservableCollection<Employee> Employees { get; private set; }

    public ObservableCollection<string> Log { get; private set; }

    public MainPageViewModel() {
      AsyncMainPageViewModel();
    }

    async void AsyncMainPageViewModel() {

      Employees = new ObservableCollection<Employee>();
      Employees.CollectionChanged += Employees_CollectionChanged;

      Log = new ObservableCollection<string>();

      // Warning: Hack alert!
      _busyPropertyName = "IsBusy"; // Starts this way for employee query

      _mgr.Fetching += (s, e) => {
        IsBusy = true;
        WriteToLog(e.Query);
      };
      _mgr.Queried += (s, e) => {
        IsBusy = false;
        _busyPropertyName = "IsOrderBusy";
        WriteToLog("Query returned " + e.Results.Cast<object>().Count());
      };

      var query = _mgr.Employees; //.Include("Orders");

      //ToDo: Add error handling
      // ToDo: Move persistence code out of the ViewModel and into a helper class
      // such as a Repository or "DataService" class
      var results = await query.ExecuteAsync();
      results.ForEach(Employees.Add);
      CurrentEmployee = Employees.FirstOrDefault();
      // Moved following line into the CurrentEmployee property
      //RaisePropertyChanged("CurrentEmployee");
    }

    void Employees_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e) {
      switch (e.Action) {
        case NotifyCollectionChangedAction.Add:
          Add(e.NewItems.Cast<Employee>());
          break;
        case NotifyCollectionChangedAction.Remove:
          Delete(e.OldItems.Cast<Employee>());
          break;
      }
    }


    private void Add(System.Collections.Generic.IEnumerable<Employee> employees) {
      // Don't bother if added employee is already in cache
      if (employees.Any(emp => emp.EntityAspect.EntityState.IsDetached())) {
        // DataForm created an employee for us
        // ToDo: Consider managing Entity Creation in the Model, not the VM
        _mgr.AddEntities(employees); // simply add to manager in bare state
      }
    }

    private void Delete(System.Collections.Generic.IEnumerable<Employee> employees) {
      // ToDo: Consider managing Delete in the model, not the VM
      employees.ForEach(emp => emp.EntityAspect.Delete());
      Save();
    }

    public void Save() {
      // ToDo: Error Handling
      _mgr.SaveChangesAsync();
    }

    private Employee _currentEmployee;
    public Employee CurrentEmployee {
      get { return _currentEmployee; }
      set { _currentEmployee = value; RaisePropertyChanged("CurrentEmployee"); }
    }

    private void WriteToLog(IEntityQuery query) {
      var returnType = query.ElementType;
      var message = (returnType == typeof(Order))
                      ? "Fetching Orders from Employee " + CurrentEmployee.FullName
                      : "Fetching " + returnType;
      WriteToLog(message);
    }

    private void WriteToLog(string message) {
      Log.Insert(0, message); // insert each message at "top" of log
    }

    private string _busyPropertyName;
    private bool _isBusy;
    public bool IsBusy {
      get { return _isBusy; }
      set {
        _isBusy = value; RaisePropertyChanged(_busyPropertyName);
      }
    }

    public bool IsOrderBusy { get { return _isBusy; } }

    #region INotifyPropertyChanged Members

    public event PropertyChangedEventHandler PropertyChanged = delegate { };

    private void RaisePropertyChanged(string property) {
      PropertyChanged(this, new PropertyChangedEventArgs(property));
    }

    #endregion
  }
}
